<?php

namespace Btravel\Objects\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Btravel\Objects\Http\Requests\ObjectRequest;
use Btravel\Objects\Interfaces\ObjectRepositoryInterface;
use Btravel\Objects\Models\Object;
use Btravel\Objects\Forms\Object as Form;

/**
 * APIController  class for object.
 */
class ObjectAPIController extends BaseController
{

    /**
     * Initialize object resource controller.
     *
     * @param type ObjectRepositoryInterface $object
     *
     * @return null
     */
    public function __construct(ObjectRepositoryInterface $object)
    {
        parent::__construct();
        $this->repository = $object;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Objects\Repositories\Criteria\ObjectResourceCriteria::class);
    }

    /**
     * Display a list of object.
     *
     * @return Response
     */
    public function index(ObjectRequest $request)
    {
        return $this->repository
            ->setPresenter(\Btravel\Objects\Repositories\Presenter\ObjectPresenter::class)
            ->paginate();
    }

    /**
     * Display object.
     *
     * @param Request $request
     * @param Model   $object
     *
     * @return Response
     */
    public function show(ObjectRequest $request, Object $object)
    {
        return $object->setPresenter(\Btravel\Objects\Repositories\Presenter\ObjectListPresenter::class);
        ;
    }

    /**
     * Create new object.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ObjectRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('objects::object.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('objects/object/' . $object->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('objects/object');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the object.
     *
     * @param Request $request
     * @param Model   $object
     *
     * @return Response
     */
    public function update(ObjectRequest $request, Object $object)
    {
        try {
            $data = $request->all();

            $object->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('objects::object.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('objects/object/' . $object->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('objects/object/' . $object->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the object.
     *
     * @param Model   $object
     *
     * @return Response
     */
    public function destroy(ObjectRequest $request, Object $object)
    {
        try {
            $object->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('objects::object.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('objects/object/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('objects/object/' . $object->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
