<?php

namespace Btravel\Objects\Policies;

use Litepie\User\Contracts\UserPolicy;
use Btravel\Objects\Models\Object;

class ObjectPolicy
{

    /**
     * Determine if the given user can view the object.
     *
     * @param UserPolicy $user
     * @param Object $object
     *
     * @return bool
     */
    public function view(UserPolicy $user, Object $object)
    {
        if ($user->canDo('objects.object.view') && $user->isAdmin()) {
            return true;
        }

        return $object->user_id == user_id() && $object->user_type == user_type();
    }

    /**
     * Determine if the given user can create a object.
     *
     * @param UserPolicy $user
     * @param Object $object
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('objects.object.create');
    }

    /**
     * Determine if the given user can update the given object.
     *
     * @param UserPolicy $user
     * @param Object $object
     *
     * @return bool
     */
    public function update(UserPolicy $user, Object $object)
    {
        if ($user->canDo('objects.object.edit') && $user->isAdmin()) {
            return true;
        }

        return $object->user_id == user_id() && $object->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given object.
     *
     * @param UserPolicy $user
     * @param Object $object
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Object $object)
    {
        return $object->user_id == user_id() && $object->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given object.
     *
     * @param UserPolicy $user
     * @param Object $object
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Object $object)
    {
        if ($user->canDo('objects.object.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given object.
     *
     * @param UserPolicy $user
     * @param Object $object
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Object $object)
    {
        if ($user->canDo('objects.object.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
